@@ -26,8 +26,6 @@
 #include <Python.h>
 #include <structmember.h>
 
-#include <stdint.h>
-
 #include "dbus_bindings-internal.h"
 #include "types-internal.h"
 
@@ -92,18 +90,18 @@ Array_tp_repr(DBusPyArray *self)
     if (!parent_repr) goto finally;
     if (!sig_repr) goto finally;
     if (variant_level > 0) {
-        my_repr = PyString_FromFormat("%s(%s, signature=%s, "
-                                      "variant_level=%ld)",
-                                      self->super.ob_type->tp_name,
-                                      PyString_AS_STRING(parent_repr),
-                                      PyString_AS_STRING(sig_repr),
-                                      variant_level);
+        my_repr = PyUnicode_FromFormat("%s(%V, signature=%V, "
+                                       "variant_level=%ld)",
+                                       Py_TYPE(&self->super)->tp_name,
+                                       REPRV(parent_repr),
+                                       REPRV(sig_repr),
+                                       variant_level);
     }
     else {
-        my_repr = PyString_FromFormat("%s(%s, signature=%s)",
-                                      self->super.ob_type->tp_name,
-                                      PyString_AS_STRING(parent_repr),
-                                      PyString_AS_STRING(sig_repr));
+        my_repr = PyUnicode_FromFormat("%s(%V, signature=%V)",
+                                       Py_TYPE(&self->super)->tp_name,
+                                       REPRV(parent_repr),
+                                       REPRV(sig_repr));
     }
 finally:
     Py_CLEAR(parent_repr);
@@ -127,11 +125,12 @@ Array_tp_new (PyTypeObject *cls, PyObject *args, PyObject *kwargs)
         variant_level = PyDict_GetItem(kwargs, dbus_py_variant_level_const);
     }
     if (variant_level) {
-        self->variant_level = PyInt_AsLong(variant_level);
-        if (PyErr_Occurred()) {
+        long new_variant_level = PyLong_AsLong(variant_level);
+        if (new_variant_level == -1 && PyErr_Occurred()) {
             Py_CLEAR(self);
             return NULL;
         }
+        self->variant_level = new_variant_level;
     }
     return (PyObject *)self;
 }
@@ -166,15 +165,42 @@ Array_tp_init (DBusPyArray *self, PyObject *args, PyObject *kwargs)
     }
 
     if (signature != Py_None) {
-        const char *c_str = PyString_AS_STRING(signature);
+        const char *c_str;
+        PyObject *signature_as_bytes;
+
+        if (
+#ifdef PY3
+            !PyUnicode_Check(signature)
+#else
+            !PyBytes_Check(signature)
+#endif
+            )
+        {
+            PyErr_SetString(PyExc_TypeError, "str expected");
+            Py_CLEAR(signature);
+            return -1;
+        }
+#ifdef PY3
+        if (!(signature_as_bytes = PyUnicode_AsUTF8String(signature))) {
+            Py_CLEAR(signature);
+            return -1;
+        }
+#else
+        signature_as_bytes = signature;
+        Py_INCREF(signature_as_bytes);
+#endif
+
+        c_str = PyBytes_AS_STRING(signature_as_bytes);
 
         if (!dbus_signature_validate_single(c_str, NULL)) {
             Py_CLEAR(signature);
+            Py_CLEAR(signature_as_bytes);
             PyErr_SetString(PyExc_ValueError,
                             "There must be exactly one complete type in "
                             "an Array's signature parameter");
             return -1;
         }
+        Py_CLEAR(signature_as_bytes);
     }
 
     tuple = Py_BuildValue("(O)", obj);
@@ -195,8 +221,7 @@ Array_tp_init (DBusPyArray *self, PyObject *args, PyObject *kwargs)
 }
 
 PyTypeObject DBusPyArray_Type = {
-    PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
-    0,
+    PyVarObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type), 0)
     "dbus.Array",
     sizeof(DBusPyArray),
     0,
@@ -300,18 +325,18 @@ Dict_tp_repr(DBusPyDict *self)
     if (!parent_repr) goto finally;
     if (!sig_repr) goto finally;
     if (variant_level > 0) {
-        my_repr = PyString_FromFormat("%s(%s, signature=%s, "
-                                      "variant_level=%ld)",
-                                      self->super.ob_type->tp_name,
-                                      PyString_AS_STRING(parent_repr),
-                                      PyString_AS_STRING(sig_repr),
-                                      variant_level);
+        my_repr = PyUnicode_FromFormat("%s(%V, signature=%V, "
+                                       "variant_level=%ld)",
+                                       Py_TYPE(&self->super)->tp_name,
+                                       REPRV(parent_repr),
+                                       REPRV(sig_repr),
+                                       variant_level);
     }
     else {
-        my_repr = PyString_FromFormat("%s(%s, signature=%s)",
-                                      self->super.ob_type->tp_name,
-                                      PyString_AS_STRING(parent_repr),
-                                      PyString_AS_STRING(sig_repr));
+        my_repr = PyUnicode_FromFormat("%s(%V, signature=%V)",
+                                       Py_TYPE(&self->super)->tp_name,
+                                       REPRV(parent_repr),
+                                       REPRV(sig_repr));
     }
 finally:
     Py_CLEAR(parent_repr);
@@ -335,11 +360,13 @@ Dict_tp_new(PyTypeObject *cls, PyObject *args, PyObject *kwargs)
         variant_level = PyDict_GetItem(kwargs, dbus_py_variant_level_const);
     }
     if (variant_level) {
-        self->variant_level = PyInt_AsLong(variant_level);
-        if (PyErr_Occurred()) {
+        long new_variant_level = PyLong_AsLong(variant_level);
+
+        if (new_variant_level == -1 && PyErr_Occurred()) {
             Py_CLEAR(self);
             return NULL;
         }
+        self->variant_level = new_variant_level;
     }
     return (PyObject *)self;
 }
@@ -373,8 +400,25 @@ Dict_tp_init(DBusPyDict *self, PyObject *args, PyObject *kwargs)
     }
 
     if (signature != Py_None) {
-        const char *c_str = PyString_AS_STRING(signature);
+        const char *c_str;
+        PyObject *signature_as_bytes;
 
+        if (!NATIVESTR_CHECK(signature)) {
+            PyErr_SetString(PyExc_TypeError, "str expected");
+            Py_CLEAR(signature);
+            return -1;
+        }
+#ifdef PY3
+        if (!(signature_as_bytes = PyUnicode_AsUTF8String(signature))) {
+            Py_CLEAR(signature);
+            return -1;
+        }
+#else
+        signature_as_bytes = signature;
+        Py_INCREF(signature_as_bytes);
+#endif
+
+        c_str = PyBytes_AS_STRING(signature_as_bytes);
         switch (c_str[0]) {
             case DBUS_TYPE_BYTE:
             case DBUS_TYPE_BOOLEAN:
@@ -397,6 +441,7 @@ Dict_tp_init(DBusPyDict *self, PyObject *args, PyObject *kwargs)
                 break;
             default:
                 Py_CLEAR(signature);
+                Py_CLEAR(signature_as_bytes);
                 PyErr_SetString(PyExc_ValueError,
                                 "The key type in a Dictionary's signature "
                                 "must be a primitive type");
@@ -405,11 +450,13 @@ Dict_tp_init(DBusPyDict *self, PyObject *args, PyObject *kwargs)
 
         if (!dbus_signature_validate_single(c_str + 1, NULL)) {
             Py_CLEAR(signature);
+            Py_CLEAR(signature_as_bytes);
             PyErr_SetString(PyExc_ValueError,
                             "There must be exactly two complete types in "
                             "a Dictionary's signature parameter");
             return -1;
         }
+        Py_CLEAR(signature_as_bytes);
     }
 
     tuple = Py_BuildValue("(O)", obj);
@@ -431,8 +478,7 @@ Dict_tp_init(DBusPyDict *self, PyObject *args, PyObject *kwargs)
 }
 
 PyTypeObject DBusPyDict_Type = {
-    PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
-    0,
+    PyVarObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type), 0)
     "dbus.Dictionary",
     sizeof(DBusPyDict),
     0,
@@ -521,20 +567,24 @@ Struct_tp_repr(PyObject *self)
     if (!sig) sig = Py_None;
     sig_repr = PyObject_Repr(sig);
     if (!sig_repr) goto finally;
+
     variant_level = dbus_py_variant_level_get(self);
+    if (variant_level < 0)
+        goto finally;
+
     if (variant_level > 0) {
-        my_repr = PyString_FromFormat("%s(%s, signature=%s, "
-                                      "variant_level=%ld)",
-                                      self->ob_type->tp_name,
-                                      PyString_AS_STRING(parent_repr),
-                                      PyString_AS_STRING(sig_repr),
-                                      variant_level);
+        my_repr = PyUnicode_FromFormat("%s(%V, signature=%V, "
+                                       "variant_level=%ld)",
+                                       Py_TYPE(self)->tp_name,
+                                       REPRV(parent_repr),
+                                       REPRV(sig_repr),
+                                       variant_level);
     }
     else {
-        my_repr = PyString_FromFormat("%s(%s, signature=%s)",
-                                      self->ob_type->tp_name,
-                                      PyString_AS_STRING(parent_repr),
-                                      PyString_AS_STRING(sig_repr));
+        my_repr = PyUnicode_FromFormat("%s(%V, signature=%V)",
+                                       Py_TYPE(self)->tp_name,
+                                       REPRV(parent_repr),
+                                       REPRV(sig_repr));
     }
 
 finally:
@@ -648,7 +698,11 @@ Struct_tp_getattro(PyObject *obj, PyObject *name)
 {
     PyObject *key, *value;
 
-    if (PyString_Check(name)) {
+#ifdef PY3
+    if (PyUnicode_CompareWithASCIIString(name, "signature"))
+        return dbus_py_variant_level_getattro(obj, name);
+#else
+    if (PyBytes_Check(name)) {
         Py_INCREF(name);
     }
     else if (PyUnicode_Check(name)) {
@@ -662,13 +716,13 @@ Struct_tp_getattro(PyObject *obj, PyObject *name)
         return NULL;
     }
 
-    if (strcmp(PyString_AS_STRING(name), "signature")) {
+    if (strcmp(PyBytes_AS_STRING(name), "signature")) {
         value = dbus_py_variant_level_getattro(obj, name);
         Py_CLEAR(name);
         return value;
     }
-
     Py_CLEAR(name);
+#endif  /* PY3 */
 
     key = PyLong_FromVoidPtr(obj);
 
@@ -686,8 +740,7 @@ Struct_tp_getattro(PyObject *obj, PyObject *name)
 }
 
 PyTypeObject DBusPyStruct_Type = {
-    PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
-    0,
+    PyVarObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type), 0)
     "dbus.Struct",
     0,
     0,
