@@ -1,33 +1,47 @@
 /* D-Bus types: implementation internals
  *
- * Copyright (C) 2006, 2007 Collabora Ltd.
+ * Copyright (C) 2006-2007 Collabora Ltd. <http://www.collabora.co.uk/>
  *
- * Licensed under the Academic Free License version 2.1
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use, copy,
+ * modify, merge, publish, distribute, sublicense, and/or sell copies
+ * of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * This library is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
  *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <Python.h>
-#include <stdint.h>
+
+/* In Python2 >= 2.6 this aliases PyString to PyBytes.  There is no PyString
+ * in Python 3, so this allows the C extension to be compilable in both Python
+ * versions.
+ */
+#include <bytesobject.h>
+
+/* In Python 2.x, we need this to define the type of PyLongObject */
+#ifndef PY3
+#include <longintrepr.h>
+#endif
 
 #include "dbus_bindings-internal.h"
 
 #ifndef DBUS_BINDINGS_TYPES_INTERNAL_H
 #define DBUS_BINDINGS_TYPES_INTERNAL_H
 
+#ifndef PY3
 extern PyTypeObject DBusPyIntBase_Type;
 DEFINE_CHECK(DBusPyIntBase)
 
@@ -35,6 +49,7 @@ typedef struct {
     PyIntObject base;
     long variant_level;
 } DBusPyIntBase;
+#endif
 
 extern PyTypeObject DBusPyLongBase_Type;
 DEFINE_CHECK(DBusPyLongBase)
@@ -55,6 +70,11 @@ typedef struct {
 extern PyTypeObject DBusPyStrBase_Type;
 DEFINE_CHECK(DBusPyStrBase)
 
+#ifdef PY3
+extern PyTypeObject DBusPyBytesBase_Type;
+DEFINE_CHECK(DBusPyBytesBase)
+#endif
+
 dbus_int16_t dbus_py_int16_range_check(PyObject *);
 dbus_uint16_t dbus_py_uint16_range_check(PyObject *);
 dbus_int32_t dbus_py_int32_range_check(PyObject *);
